%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Validation script for ground precision by flightline and by polygon

%Inputs: A text file for las pts extracted from each validation polygon
%with xyzpc attributes
%
%Ouput: RMSE by flight line and RMSE by polygon
%
%Parameters to set: The percentage of points to sample (in large datasets the
%regression will not work). Default = 20% (0.2)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% percent of dataset to include in precision calculation
subsample = 0.4;

%storage location of text files containing lidar point info in 5 xyzpc
%format
base_path = '..\ground_precision_val\';

%get list of text files in the base dir
dir_txt = dir(strcat(base_path,'*.txt'));

%construct an error matrix where there is one row for each flightline represented in
%each polygon. Compute mean squared error using multivariate regression.
full_error_mat = [];

%loop through each text file (polygon)
for i = 1:size(dir_txt,1)
    
    fileName = dir_txt(i).name;
    
    %check if text file has data
    if dir_txt(i).bytes > 0 
        M_full = dlmread(strcat(base_path,fileName));
        
        %need to subsample to maintain full rank matrix
        M_rand = rand(length(M_full),1);
        rand_idx = find(M_rand < subsample); %keep x% of data
        M = M_full(rand_idx,:);
        
        %find unique flight lines in this polygon
        lines = unique(M(:,4));
        %loop through each flight line represented within each polygon
        for j=1:length(lines)
            line_idx = find(M(:,4)==lines(j));
            constants = ones(length(line_idx),1);
            X=horzcat(constants, M(line_idx,1:2));
            z=M(line_idx,3);
            [b,bint,r] = regress(z,X); %r is vector of residuals
            error_mat(j,1) = i;
            error_mat(j,2) = lines(j);
            %need this count to weight average polygon error
            error_mat(j,3) = length(line_idx); 
            %calculate RMSE
            error_mat(j,4) = sqrt(mean(r.^2));
            error_mat(j,5) = std(r);
        end
    full_error_mat = vertcat(full_error_mat,error_mat);
    clear error_mat
        
    end
    
end


%error aggregated by flightline (across polygons)
f_lines = unique(full_error_mat(:,2));
for i = 1:length(f_lines);
    line_idx = find(full_error_mat(:,2)==f_lines(i));
    poly_line_count = full_error_mat(line_idx,3);
    poly_line_err = full_error_mat(line_idx,4);
    %average error weighted by number of points per flight line (using inner
    %product)
    line_err(i,2) = (poly_line_count'*poly_line_err)/sum(poly_line_count);
    line_err(i,1) = f_lines(i);
end

%error aggregated by polygon (across flightlines)
polys = unique(full_error_mat(:,1));
for i = 1:length(polys);
    poly_idx = find(full_error_mat(:,1)==polys(i));
    poly_line_count = full_error_mat(poly_idx,3);
    poly_line_err = full_error_mat(poly_idx,4);
    %average error weighted by number of points per flight line (using
    %inner product)
    poly_err(i,2) = (poly_line_count'*poly_line_err)/sum(poly_line_count);
    poly_err(i,1) = polys(i);
end

%report line and poly error
line_err
poly_err


%visualize the fit between the points and a fit plane
scatter3(X(:,2),X(:,3),z,30,'filled')
hold on
x1fit = min(X(:,2)):0.1:max(X(:,2));
x2fit = min(X(:,3)):0.1:max(X(:,3));
[X1FIT,X2FIT] = meshgrid(x1fit,x2fit);
YFIT = b(1) + b(2)*X1FIT + b(3)*X2FIT;
mesh(X1FIT,X2FIT,YFIT)
