function cmm = max_filter_variable_window(chm,seg_prog)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%This code accepts a canopy height model of any size as input (and
%optionally a parameter to indicate computation progress).

%It is recommended that the input chm here is relatively small (i.e. run on
%individual segments rather than a full scene). Performance, partly because
%this needs to run on every pixel and partly because of the use of
%imdilate, is not optimal.

%Output: a canopy maxima model of the same dims as the input CHM.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%get dimensions of incoming chm
m_dim = size(chm,1);
n_dim = size(chm,2);
bw_im = zeros(size(chm));
cmm = zeros(size(chm));

%set the model image upon which to base the variable window
%user needs to determine the way in which this will be calculated. Below I
%show a linear example and a 2nd degree polynomial based the relationship
%between tree height and crown width in Santa Barbara using field data.
win = 0.56  * chm + 1.82; %linear
%win = -0.0062*(chm.^2) + 0.9171*chm + 0.159; %poly2

%convert to a "radius image". Each pixel in win_rad contains the value of
%the search radius to be used to create the variable window.
win_rad = (round(win./2)).*4;


%Purpose of this loop:
%Iterate through image using linear indexing (outperformed coordinate
%indexing significantly). Create a variably sized mask based on the radius
%image at that address. Use the mask to grab the neighborhood max value
%and assign to CMM at the address.
for j = 1:m_dim    
    for k=1:n_dim
        
       %find linear position of seed px
       lin_idx = sub2ind(size(chm),j,k); 
       bw_im(lin_idx) = 1;
       
       %set the window size using a structured element
       %Note: setting SE using "ones()" is much faster than creating a
       %true disc shape using strel but probably less accurate
       if win_rad(lin_idx) > 0
           SE = ones(win_rad(lin_idx),win_rad(lin_idx)); 
       else
           SE = ones(1,1);
       end
       
       %dilate to create a search neighborhood within which the maximum
       %value will be found
       bw_th = imdilate(bw_im,SE);
    
       %grab the max chm value that is within the SE and assign
       %it to the CMM at the address in question
       cmm(lin_idx) = max(chm(bw_th==1));
              
       %reset bw_im
       bw_im(lin_idx)=0;
    end
    
end

%print progress if input parameter provided
if nargin == 2
    fprintf('Seg: %u\n',seg_prog)
end

return





    