%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Control file for all watershed-segmentation related processes
%
%NOTE:  This code is for reference purposes only. You should not assume
%that this will work for your specific imagery. Feel free to edit as
%needed. Please cite Alonzo et al. (2014).
%
%Required functions (beyond standard Matlab image processing functionality)
    %watershedshape.m
    %cmmWrap.m
    %max_filter_variable_window.m
    %watershedsurface1.m
    %multicore suite: http://www.mathworks.com/matlabcentral/fileexchange/13775-multicore-parallel-processing-on-multiple-cores
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Input imagery:  Assumes gridded layers with some previous classification
%(for instance: point cloud classification with gridded export from
%lastools)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%High vegetation geotiff with spatial reference
addpath('..\surface_models');
[veg_high,R_veg] = geotiffread('..\surface_models\veg_high_fill_sev_full.tif');
veg_high = double(veg_high);

%High unclassified imagery (along with separate tfw worldfile)
file_data= '..\surface_models\uncl_high_full_scene.tif';
uncl_high = double(imread(file_data)); 
worldFileName = getworldfilename(file_data);
uncl_high_tfw = worldfileread(worldFileName, 'planar', size(uncl_high));

%Read in building data with separate tfw worldfile
file_data= '..\surface_models\bldg_elev_full_scene.tif';
bldg_elev = double(imread(file_data)); 
worldFileName = getworldfilename(file_data);
bldg_elev_tfw = worldfileread(worldFileName, 'planar', size(bldg_elev));
%pad the building layer b/c the size is not the same as the others (though
%the alignment is fine)
bldg_elev_pad = zeros(size(veg_high));
bldg_elev_pad(1:size(bldg_elev,1),1:size(bldg_elev,2)) = bldg_elev;
bldg_elev = bldg_elev_pad;

%bring in canopy class (binary image) with worldfile
file_data= '..\surface_models\canopy_class_full.tif';
canopy_class = logical(imread(file_data)); 
worldFileName = getworldfilename(file_data);
canopy_class_tfw = worldfileread(worldFileName, 'planar', size(canopy_class));

%This section logically combines high veg, unclassified imagery, and buildings that
%exist within the existing canopy classification. Purpose: use the binary
%canopy layer as the gold standard and reclassify all layers based on it.
veg_high_log = logical(veg_high);
veg_high_canopy_log = veg_high_log.*canopy_class;
uncl_high_log = logical(uncl_high);
bldg_elev_log = logical(bldg_elev);
bldg_elev_canopy_log = bldg_elev_log.*canopy_class;
bldg_elev_canopy = bldg_elev.*bldg_elev_canopy_log;
just_uncl_log = ~veg_high_log.*(uncl_high_log.*canopy_class);
just_uncl = just_uncl_log.*uncl_high;
just_bldg_log = ~(veg_high_log | uncl_high_log).*bldg_elev_canopy_log;
just_bldg = just_bldg_log.*bldg_elev;
veg_high_canopy = veg_high.*veg_high_canopy_log;
veg_high = veg_high_canopy + just_uncl + just_bldg; %final canopy layer

%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%basic morphological operations to clean binary canopy layer
% (took about 2 min to run on Santa Barbara scene)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%convert to binary image
veg_high_bw = im2bw(veg_high,0.05);

%morphological opening and closing 
veg_high_O = bwmorph(veg_high_bw,'open');
veg_high_C = bwmorph(veg_high_O,'close');

%further cleaning
veg_high_T = bwmorph(veg_high_C,'thicken',1);
veg_high_F = bwmorph(veg_high_T,'fill');
veg_high_F = bwmorph(veg_high_F,'majority'); 

%a final elimination of canopy less than 16 px (=1sq meter at 0.25 m res)
veg_high_O2 = bwareaopen(veg_high_F,16);

%Thickening required because both watershed-shape and
%watershed-surf progressively thin segments because segment boundaries have
%1-pixel thickness
veg_high_O2 = bwmorph(veg_high_O2,'thicken',2); 

%cleaned up canopy height model (CHM)
veg_high_clean = medfilt2((veg_high_O2.*veg_high),[3 3]);
%binary image (for use in watershedshape and future cleaning)
veg_high_bin = veg_high_O2;


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Watershed segmentation baed on distance transform
%Required functions:
%   watershedshape.m
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%w-shape (took about 13 min to run on Santa Barbara scene)
[w_shape,w_shape_lines] = watershedshape(...
        veg_high_bin,50);
    
%Display results
% shape_over = imoverlay(mat2gray(veg_high), w_shape_lines,[.3 1 .3]);
% imtool(shape_over);
   
%clear stuff because memory starts to get tight
clear veg_high_FULL
clear veg_high_C
clear veg_high_T
clear veg_high_O

%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Prepare segments for processing to CMM and watershed seg

%The main purpose of this code block is to put every segment (as created
%from watershedshape) into its own cell so that it can be processed to CMM
%indpendent of the rest of the image. This allows for a vast reduction in
%processing time for two reasons:  1. CMM generation is only happening
%where canopy exists (<40% of most urban scenes) and 2. Parallelization
%using multicore.m is now easy to implement and allows for an arbitrary
%decrease in processing time depending on how many cores are at your
%disposal.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%pad the entire output of watershedshape to avoid border issues
w_shape_pad = padarray(w_shape, [10 10]); 
w_shape_pad = bwlabel(w_shape_pad); %ensure this is label-style
veg_high_clean_pad = padarray(veg_high_clean, [10 10]);

%sort and count all of the watershedshape segments as labeled by bwlabel
seg_ids = sortrows(unique(w_shape_pad));
seg_ids = seg_ids(2:end); %exclude zero
n = length(seg_ids); %-1 to exclude zero
    
%access area, bounding box extents, and bb ul corner coords
seg_props = regionprops(w_shape_pad,'Area','BoundingBox');

%This loop does two things: 
%1. Clip segments from the full image based on bounding box, 
%2. eliminate vegetation from other segments that has leaked
%   into the bounding box of the segment of interest, 

   for i = 1:n
    %extract image subset
        m_ul(i) = floor(seg_props(seg_ids(i)).BoundingBox(2)) - 3; %"-3" = segment padding
        n_ul(i) = floor(seg_props(seg_ids(i)).BoundingBox(1)) - 3;
        m_ll(i) = m_ul(i) + seg_props(seg_ids(i)).BoundingBox(4) + 6; %"+6" = segment padding
        n_ll(i) = n_ul(i) + seg_props(seg_ids(i)).BoundingBox(3) + 6; 

        %cut out the clean image and the logical sub to exclude any other
        %segments that are leaking into the bounding box. Veg from other
        %segments leaking in will cause problems when calculating CMM.
        veg_high_clean_pad_logical_sub{i} = w_shape_pad(m_ul(i):m_ll(i),n_ul(i):n_ll(i))==seg_ids(i);
        veg_high_clean_pad_sub{i} = veg_high_clean_pad(m_ul(i):m_ll(i),n_ul(i):n_ll(i)).*...
            veg_high_clean_pad_logical_sub{i};
   end
 
   
   %This loop fills the parameterCell in preparation for parallel
   %processing of CMM generation. The nested loop holds each segment while
   %the outer loop holds groups of segments (number of groups = subdiv).
   %Purpose: CMM parallelization works much better via multicore if the
   %packets sent out to slaves are large.
        count = 0;
        subdiv = 36;  %number of packets that will be divvied to slaves
        sub_size = floor(length(veg_high_clean_pad_sub)/subdiv);
        sub_remainder = length(veg_high_clean_pad_sub)-(sub_size * subdiv);
        for i = 1:subdiv
            
            for j = 1:length(veg_high_clean_pad_sub)/subdiv
                chm_sub{j} = veg_high_clean_pad_sub{count+j};
            end
            
            count = count + j;
            parameterCell{i} = {chm_sub};
            
            clear chm_sub
        
        end

%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Run the parallel process for creating the per-segment CMM
%Parallelization needs:
% Open an additional matlab window for each additional slave process up to
% number-of-available-cores - 1
% run startmulticoreslave.m on each

%Required functions:
%   cmmWrap.m (if you're grouping segments in larger sets)
%   max_filter_variable_window.m
%   the suite of multicoremaster functions

%DO NOT RUN THIS BLOCK IF YOU ALREADY HAVE A CMM CREATED
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%parallelize the processing of the CMM  
%fprintf('About to start CMM for: %u\n',plots) 
tic
cmm_out = startmulticoremaster(@cmmWrap,...
    parameterCell);
toc_cmm = toc;
%fprintf('Completed CMM for: %u\n',plots)

%cell storing packages of processed CMMs
cmm_storage = cmm_out;

%IF YOU USED cmmWrap: NEED TO BREAK SEGS OUT OF THEIR packages
cmm_break = [];
for i = 1:length(cmm_out)
    cmm_break = vertcat(cmm_break, cmm_out{i});
end
cmm_out = cmm_break; %reassign for downstream processing


%define output images
w_surf_plot_pad = zeros(size(veg_high_clean_pad));
cmm_plot_pad = zeros(size(veg_high_clean_pad));

%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Run the CMM-based watershed segmentation
%Required functions:
%   watershedsurface1.m
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%gather data about segments to check to see whether further segmentation should occur
stats_bw = regionprops(logical(w_shape_pad),'eccentricity','area');
stats_veg_high = regionprops(logical(w_shape_pad), ...
veg_high_clean_pad,'MeanIntensity');

%Run the CMM-based watershed segmentation and restitch CMM and final
%segments back into full images

for i=1:n

    %SWAP COMMENTS WHEN ACCESSING PRE-CREATED CMM
    %reduce speckle of cmm with median filter
    cmm_med3 = medfilt2(cmm_out{i}, [3 3]);
    %cmm_med3 = medfilt2(cmm_storage{i}, [3 3]);
    
    %output should not have grown due to max filtering; re-constrain 
    cmm_constrained{i} = cmm_med3.*veg_high_clean_pad_logical_sub{i};

   %only send segment for further segmentation if it's large and not very
   %circular. These particular values were determined empirically based on 
   %a sampling of validation segments.
   if stats_bw(i).Area < 1000 && stats_bw(i).Eccentricity < 0.7
        %more_seg(i) = 0;
        w_surf{i} = logical(cmm_constrained{i});
    else
        more_seg(i) = 1;
        [w_surf{i},w_surf_lines{i},w_surf_maxes{i}] = ...
        watershedsurface1(cmm_constrained{i},more_seg(i));
    end

    %more_seg(i) = 1;

    %iteratively restitch the segments into the full scene
    w_surf_plot_pad(m_ul(i):m_ll(i),n_ul(i):n_ll(i)) = ...
        w_surf_plot_pad(m_ul(i):m_ll(i),n_ul(i):n_ll(i)) + ...
        double(w_surf{i});

    cmm_plot_pad(m_ul(i):m_ll(i),n_ul(i):n_ll(i)) = ...
        cmm_plot_pad(m_ul(i):m_ll(i),n_ul(i):n_ll(i)) + ...
        cmm_constrained{i};
end



%ditch padding on image
cmm_plot = cmm_plot_pad(11:size(cmm_plot_pad,1)-10,...
        11:size(cmm_plot_pad,2)-10);
%ditch padding on image
w_surf_plot = w_surf_plot_pad(11:size(w_surf_plot_pad,1)-10,...
        11:size(w_surf_plot_pad,2)-10);

   
   
 %%
 %export segments

export_dir = '..\full_scene\';

%w-shape output
%NOTE: export the solid polygon and NOT the outline
fileName = horzcat('w-shape_full_scene_130812.tif');
geotiffwrite(horzcat(export_dir,fileName),logical(w_shape),R_veg,...
    'CoordRefSysCode','EPSG:26911');

%w-surf output
%NOTE: export the solid polygon and NOT the outline
fileName = horzcat('w-surf_full_scene_130812.tif');
geotiffwrite(horzcat(export_dir,fileName),logical(w_surf_plot),R_veg,...
    'CoordRefSysCode','EPSG:26911');

%cmm output
fileName = horzcat('cmm_lin_130812.tif');
geotiffwrite(horzcat(export_dir,fileName),cmm_plot,R_veg,...
    'CoordRefSysCode','EPSG:26911');


%output the veg_high_clean layer
fileName = horzcat('veg_high_clean_130812.tif');
geotiffwrite(horzcat(export_dir,fileName),veg_high_clean,R_veg,...
    'CoordRefSysCode','EPSG:26911');

%output the veg_high_bin binary layer
fileName = horzcat('veg_high_bin_130812.tif');
geotiffwrite(horzcat(export_dir,fileName),veg_high_bin,R_veg,...
    'CoordRefSysCode','EPSG:26911');

    
  
    