function [L_fore, L_lines] = watershedshape(bw_image, exclusion_param)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Purpose: Segment a binary image based on inverse distance transform

%Inputs: a binary image and an exclusion parameter to mask out areas that are too far 
%from any edges to be reliably segmented. 
%   For example: If there are large swaths of closed-canopy forest, there
%   is no basis on which to conduct shape-based segmentation. Allowing the
%   process to run in these areas frequently results in meaningless
%   segments.
%   Values:  I use "50" which means that areas >50 pixels from an edge will
%   be masked out of this analysis. This number is based on image
%   resolution and the scale of vegetated areas in the study area.

%Outputs: 
%   L_fore: foreground watershed segments
%   L_lines: the watershed boundary lines

%NOTE:  This code is for reference purposes only. You should not assume
%that this will work for your specific imagery. Feel free to edit as
%needed. Please cite Alonzo et al. (2014).
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%check input arguments
if nargin < 2
    exclusion_param = 50;
elseif nargin < 1
    error('Too few arguments for watershedshape.');
end

%shape-based watershed
D = -bwdist(~bw_image);
include = D > -exclusion_param; %don't use watershedshape on forested stands

%find mins of the complemented inverse dist image and mark them
D_min = imextendedmin(D,1).*include;
D_imp = imimposemin(D,D_min);

%watershed transform
D_imp(D==0)=-Inf;
L = watershed(D_imp);

%cut out the background segments and clean tiny segments
L_fore = immultiply(L,bw_image);
L_fore_bw_clean = bwareaopen(L_fore,16);
L_fore = immultiply(L_fore,L_fore_bw_clean);

%generate outlines from L (for display purposes)
L_lines = L==0;


return
