function [w_surf, w_surf_lines] = ...
    watershedsurface1(image,more_seg, shape_result)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Purpose:
    %This function takes a surface model and optionally boundaries that
    %have already been defined by a previous segmentation (e.g. shape-based
    %watershed) and computes the 2.5D watershed transformation. It returns
    %the label matrix (w_surf),and the combined lines from this watershed 
    %and the input segments (w_surf_lines).
    
%Inputs:
    %image = the surface model to be segmented
    %more_seg = flag. 0 = do not actually run segmentation
    %shape_result = lines from previous segmentation. Not really relevant
    %when running this code on individual segments but this code can be run
    %on larger images too. Then shape_result will allow for retention of
    %original boundaries in addition to the new boundaries.
    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    %adaptive smoothing
    image = wiener2(image,[3 3],1);
    
    %%set up outlines and layers
    veg_high_bw = im2bw(image,0.05);

    %set defaults for optional arguments    
    if nargin < 3
        %if no previous segments supplied the boundary of the binary image
        %will be considered the shape segment
        shape_result = ~bwperim(veg_high_bw,8);
    elseif nargin < 1
        error('Too few arguments for watershedsurface1.');
    end
    
    %if more_seg flag is 0 then this segmentation will not occur
    if more_seg == 0;
        w_surf = bwlabel(veg_high_bw);
        w_surf_lines = ~shape_result;
        return
    end
    
   
    %place boundaries from shape-based watershed on image
    L_0 = shape_result == 0;
    image(L_0) = 0;


    %find mins 
    veg_comp = imcomplement(image);
    veg_min = imregionalmin(veg_comp);
  
    %clean up the mins
    veg_min_F = imfill(veg_min,'holes');
    veg_min_O = bwareaopen(veg_min_F,2);
    veg_min_S = bwmorph(veg_min_O,'shrink',Inf);
    

    %impose mins
    veg_imp = imimposemin(veg_comp,veg_min_S);
    veg_imp(~shape_result) = Inf;
    %set the background stuff to neg inf
    veg_imp(veg_imp>0 & veg_imp < 10000)=-Inf;
    %only find mins where -Inf has been imposed
    veg_imp = imhmin(veg_imp,100);

    %compute watershed
    veg_L = watershed(veg_imp,8);
%     figure, imagesc(veg_L);

    %cut out the background segments and clean tiny segments
    w_surf = immultiply(veg_L,veg_high_bw);
    L_fore_bw_clean = bwareaopen(w_surf,16);
    w_surf = immultiply(w_surf,L_fore_bw_clean);
 

    %segment boundaries
    w_surf_lines = veg_L == 0;
  

   
return








