function [N,S,E,W, diam, count] = widthatheight(tree_E,tree_N,tree_H, ht_in_crown, slice_sz, ...
    bin_size)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %Purpose:
    %This function takes a the northings, eastings, and height triplets for
    %a crown or crown segment and computes the width of the crown at
    %user-defined heights. 
    %This is helpful in quantifying tree form.

    % Required arguments:
    % tree_E = (m,1) vector of eastings
    % tree_N = (m,1) vector of northings (of same points)
    % tree_H = (m,1) vector of heights (of same points)
    % ht_in_crown = height at which you want to find width
    % 
    % Optional args:
    % slice_sz = slice size (half the total up-down search distance in which to 
    %     locate points (above and below ht_in_crown), default = 0.5 m
    % bin_size = bin size over which to aggregate points, default = 0.25 m
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


    %set defaults for optional arguments    
    if nargin <=5
        bin_size = 0.25;
    elseif nargin <= 4
        slice_sz = 0.5;
    elseif nargin <=3
        error('Too few arguments for widthatheight.');
    end
        
     
 %  index points within slice_sz of the ht_in_crown
    idx_height = find(tree_H > ht_in_crown-slice_sz & tree_H <  ht_in_crown+slice_sz);  
    count = length(idx_height);
   
    if ~isempty(idx_height)
        %subsets of points near median height    
        tree_E_sub = tree_E(idx_height);
        tree_N_sub = tree_N(idx_height); 

        %binning EW
        bins_EW = min(tree_E_sub):bin_size:max(tree_E_sub); 

        if length(bins_EW) >= 3
            [avg,num,std]=bindata(tree_E_sub,tree_N_sub,bins_EW);
            nan_idx = find(isnan(num));
            EW_num = num; EW_num(nan_idx) = 0; 

            %find the east and west extents of crown at ht_in_crown
            furthest_east_idx = find(cumsum(EW_num)>(0.99*sum(EW_num)),1,'first');
            E = (bin_size * furthest_east_idx) + min(tree_E_sub);
            furthest_west_idx = find(cumsum(EW_num)>(0.01*sum(EW_num)),1,'first');
            W = (bin_size * furthest_west_idx) + min(tree_E_sub);
        else
            warning('Cannot create viable EW line from number of points');
            E = min(bins_EW);
            W = max(bins_EW);
        end


        %binning NS
        bins_NS = min(tree_N_sub):bin_size:max(tree_N_sub); 

        if length(bins_NS) >=3
            [avg,num,std]=bindata(tree_N_sub,tree_N_sub,bins_NS);
            nan_idx = find(isnan(num));
            NS_num = num; NS_num(nan_idx) = 0; %#ok<FNDSB>

            %NS_avg = avg; NS_avg(nan_idx) = 0;
            %NS_std = std; NS_std(nan_idx) = 0;

            %find the north and south extents of crown at ht_in_crown
            furthest_north_idx = find(cumsum(NS_num)>(0.99*sum(NS_num)),1,'first');
            N = (bin_size * furthest_north_idx) + min(tree_N_sub);
            furthest_south_idx = find(cumsum(NS_num)>(0.01*sum(NS_num)),1,'first');
            S = (bin_size * furthest_south_idx) + min(tree_N_sub);
        else
            warning('Cannot create viable NS line from number of points');
            N = min(bins_NS);
            S = max(bins_NS);
        end
    else
        %if there are no points in this slice assign a zero length
        %line to the midpoint of the segment
         warning('Cannot create viable NS line from number of points');
            N = (min(tree_N)+max(tree_N))/2;
            S = N;
            E = (min(tree_E)+max(tree_E))/2;
            W = E;
    end
    
    %compute average diam at the given height
    diam = ((N-S)+(E-W))/2;
    